//
//  AlbumArtBroker.cs
//
//  Copyright (c) Lukas Lipka, 2007.
//

using System;
using System.IO;
using System.Net;
using System.Web;
using System.Xml;
using System.Collections;
using System.Collections.Generic;

namespace Banshee.Webcast.Brokers {
	
	public class AlbumArtBroker : IBroker {
		
		private const string ALBUM_URL = "http://ws.audioscrobbler.com/1.0/album/{0}/{1}/info.xml";

		public AlbumArtBroker ()
		{
		}

		private static string RequestXml (string url)
		{
			HttpWebRequest request = (HttpWebRequest)WebRequest.Create (url);
			request.UserAgent = "Banshee.Web.MediaCenter";
			request.KeepAlive = false;
			request.Timeout = 5000;
			
			HttpWebResponse response = null;

			try { 
				response = (HttpWebResponse)request.GetResponse ();
			} catch (Exception e) {
				Log.Write (e.Message);
				return null;
			}

			Stream stream = response.GetResponseStream ();
			
			StreamReader reader = new StreamReader (stream);
			string content = reader.ReadToEnd ();
			reader.Close ();
			
			return content;
		}

		private static string EscapeFu (string text)
		{
			text = text.Replace ("/", "%2F");
			text = text.Replace (",", "%2C");
			text = text.Replace ("&", "%26");
			text = text.Replace ("?", "%3F");
			return HttpUtility.UrlEncode (text);
		}
				
		public void HandleRequest (HttpListenerContext context)
		{
			HttpListenerRequest request = context.Request;
			HttpListenerResponse response = context.Response;

			string artist = EscapeFu (request.QueryString ["artist"]);
			string album = EscapeFu (request.QueryString ["album"]);
			string url = String.Format (ALBUM_URL, artist, album);

			Log.Debug (this, "Request album art ({0} - {1})", 
				   request.QueryString ["artist"],
				   request.QueryString ["album"]);

			string xml = RequestXml (url);
			
			if (!String.IsNullOrEmpty (xml)) {
				XmlDocument album_data = new XmlDocument ();
				album_data.LoadXml (xml);
				
				XmlNode cover_art_node = album_data.SelectSingleNode ("/album/coverart/small");
				
				Log.Debug (this, "Resolved album art: " + cover_art_node.InnerText);
				
				byte[] buffer = System.Text.Encoding.UTF8.GetBytes (cover_art_node.InnerText);
				response.OutputStream.Write (buffer, 0, buffer.Length);
			}

			response.OutputStream.Close ();
		}

		public string Name { 
			get { return "Album Art request broker"; }
		}

		public string Path { 
			get { return "/albumart/*"; }
		}
	}
}
