//
//  HtmlBroker.cs
//
//  Copyright (c) Lukas Lipka, 2007.
//

using System;
using System.IO;
using System.Net;
using System.Collections;
using System.Collections.Generic;

namespace Banshee.Webcast.Brokers {
	
	public class HtmlBroker : IBroker {
		
		private static string[] index_files = { "index.htm", "index.html" };

		public HtmlBroker ()
		{
		}
		
		public void HandleRequest (HttpListenerContext context)
		{
			HttpListenerRequest request = context.Request;
			HttpListenerResponse response = context.Response;

			string path = null;
			string page = null;

			if (request.Url.LocalPath.LastIndexOf ('/') > 0) {
				int start_index = request.Url.LocalPath.IndexOf ('/');
				int count = request.Url.LocalPath.LastIndexOf ('/') - start_index;
				path = request.Url.LocalPath.Substring (start_index + 1, count);
			}

			if (request.Url.LocalPath.LastIndexOf ('/') < (request.Url.LocalPath.Length - 1)) {
				page =  request.Url.LocalPath.Substring (request.Url.LocalPath.LastIndexOf ('/') + 1);
			}

			// FIXME: Get our virtual to physical remapping
			// code in shape!
			//string local_path = ResolveMap (path);
			if (path == null) {
				path = Directory.GetCurrentDirectory ();
			}

			if (page == null) {
				foreach (string index_file in index_files) {
					string local_index = System.IO.Path.Combine (path, index_file);

					if (File.Exists (local_index)) {
						page = index_file;
						break;
					}
				}
			}

			string local_file = System.IO.Path.Combine (path,  page);

			if (File.Exists (local_file)) {
				int bytes_read = 0;
				byte[] buffer = new byte [4096];
				
				using (FileStream stream = new FileStream (local_file, FileMode.Open)) {
					response.ContentLength64 = stream.Length;
				
					while ((bytes_read = stream.Read (buffer, 0, 4096)) > 0) {
						response.OutputStream.Write (buffer, 0, bytes_read);
					}
				}
			} else {
				// FIXME: 404 error :-)
			}
			
			response.Cookies = request.Cookies;
			response.OutputStream.Close();
		}

		public string Name { 
			get { return "HTML request broker"; }
		}		

		public string Path { 
			get { return "/*"; }
		}
	}
}
