//
//  JsonBroker.cs
//
//  Copyright (c) Lukas Lipka, 2007.
//

using System;
using System.IO;
using System.Net;
using System.Collections;
using System.Collections.Generic;

namespace Banshee.Webcast.Brokers {
	
	public class JsonBroker : IBroker {
		
		private BansheeProxy proxy = null;
		private object model_lock = null;

		public JsonBroker ()
		{
			this.proxy = new BansheeProxy ();
			this.model_lock = new object ();
		}	       		

		private string ParseRequestType (string path)
		{
			// Example path: "/json/albums/"

			// 1. Remove the "/json/" part with a constant
			// length of 6 characters.
			string request_type = path.Remove (0, 6);

			// 2. Check to see if there is a trailing slash,
			// if so remove everything after it, otherwise we
			// already have the request.
			int index = request_type.IndexOf ('/');

			if (index > 0)
				request_type = request_type.Substring (0, index);
			
			return request_type;
		}

		public void HandleRequest (HttpListenerContext context)
		{
			HttpListenerRequest request = context.Request;
			HttpListenerResponse response = context.Response;

			string path = request.Url.LocalPath;
			string query = request.Url.Query;
			string request_type = ParseRequestType (path);

			string[] properties = null;

			// A query string for albums looks like this:
			//     (?artists=Lukas&artists=Joe)
			// 
			// Here we extract the request type (albums) and
			// properties passed with the request (artist = ...)
			if (query.Length > 0) {
				int index = query.IndexOf ('&');
				
				if (index > 0) {
					string props = query.Substring (index + 1);
					properties = props.Split ('&');
				}
			}

			string json_data = null;

			lock (model_lock) {
				switch (request_type) {
				case "artists":
					json_data = proxy.GetArtists ();
					break;
					
				case "albums":
					json_data = proxy.GetAlbums (properties);
					break;
					
				case "tracks":
					json_data = proxy.GetTracks (properties);
					break;					
				}
			}

			if (json_data != null) {
				byte[] buffer = System.Text.Encoding.UTF8.GetBytes (json_data);
				response.OutputStream.Write (buffer, 0, buffer.Length);
			}

			response.OutputStream.Close ();
		}

		public string Name { 
			get { return "JSON broker"; }
		}		

		public string Path { 
			get { return "/json/*"; }
		}
	}
}
