//
//  BansheeProxy.cs
//
//  Copyright (c) Lukas Lipka, 2007.
//

using System;
using System.IO;
using System.Text;
using System.Collections.Generic;

using Banshee.Base;
using Banshee.Sources;
using Banshee.Library;
using Banshee.ServiceStack;
using Banshee.Collection;
using Banshee.Collection.Database;

using Banshee.Webcast.Json;

namespace Banshee.Webcast {

	public class BansheeProxy {

		private LibrarySource library = null;
		private JsonSerializer serializer = null;

		static BansheeProxy ()
		{
			Banshee.ServiceStack.Application.Run();
		}

		public BansheeProxy ()
		{
			this.library = ConnectToLibrary ();
			this.serializer = new JsonSerializer ();
			
			Log.Debug (this, "Library '{0}' - {1} songs", library.Name, library.Count);
		}

		private LibrarySource ConnectToLibrary ()
		{
			foreach (Source source in ServiceManager.SourceManager.Sources) {
				if (source is LibrarySource)
					return (LibrarySource)source;
			}
			
			return null;
			
			//ISource library = DBusServiceManager.FindInstance<ISource>("/SourceManager/LibrarySource");
			//return (LibrarySource)library;
		}

		public string GetArtists ()
		{
			List<JArtist> artists = new List<JArtist> ();
			ArtistListDatabaseModel artist_model = (ArtistListDatabaseModel)library.ArtistModel;

			for (int i = 0; i < artist_model.Rows; i++) {
				ArtistInfo artist = artist_model.GetValue (i);
				artists.Add (new JArtist (i.ToString (), artist.Name));
			}

			string json_data = serializer.Serialize (artists.ToArray ());
			return json_data;
		}

		public string GetAlbums (string[] properties)
		{
			// Construct our artist filter to get only albums relevant
			// to the artist we requested.
			List<ArtistInfo> artist_filter = new List<ArtistInfo> ();
			ArtistListDatabaseModel artist_model = (ArtistListDatabaseModel)library.ArtistModel;
			
			foreach (string property in properties) {
				int id = Int32.Parse (property.Substring (property.IndexOf ('=') + 1));
				ArtistInfo artist = artist_model.GetValue (id);
				artist_filter.Add (artist);
			}

			// The album query itself
			List<JAlbum> albums = new List<JAlbum> ();
			AlbumListDatabaseModel model = (AlbumListDatabaseModel)library.AlbumModel;
			model.ArtistInfoFilter = artist_filter;

			for (int i = 0; i < model.Rows; i++) {
				AlbumInfo album = model.GetValue (i);
				albums.Add (new JAlbum (i.ToString (), album.Title));
			}

			string json_data = serializer.Serialize (albums.ToArray ());
			return json_data;
		}

		public string GetTracks (string[] properties)
		{
			// Construct artist filter
			List<ArtistInfo> artist_filter = new List<ArtistInfo> ();
			ArtistListDatabaseModel artist_model = (ArtistListDatabaseModel)library.ArtistModel;
			
			foreach (string property in properties) {
				if (property.StartsWith ("album"))
					continue;

				int id = Int32.Parse (property.Substring (property.IndexOf ('=') + 1));
				ArtistInfo artist = artist_model.GetValue (id);
				artist_filter.Add (artist);
			}

			// Construct album filter
			List<AlbumInfo> album_filter = new List<AlbumInfo> ();
			AlbumListDatabaseModel album_model = (AlbumListDatabaseModel)library.AlbumModel;
			album_model.ArtistInfoFilter = artist_filter;
			
			foreach (string property in properties) {
				if (property.StartsWith ("artist"))
					continue;

				int id = Int32.Parse (property.Substring (property.IndexOf ('=') + 1));

				if (id == 0) // All albums
					break;

				AlbumInfo album = album_model.GetValue (id);
				album_filter.Add (album);
			}
			
			List<JTrack> tracks = new List<JTrack> ();
			TrackListDatabaseModel track_model = (TrackListDatabaseModel)library.TrackModel;
			track_model.AlbumInfoFilter = album_filter;
			track_model.ArtistInfoFilter = artist_filter;

			for (int i = 0; i < track_model.Rows; i++) {
				TrackInfo track = track_model.GetValue (i);
				tracks.Add (new JTrack (i.ToString (), track.TrackNumber.ToString (), track.ArtistName, track.AlbumTitle, track.TrackTitle, track.Duration.ToString ()));
			}
									   
			string json_data = serializer.Serialize (tracks.ToArray ());
			return json_data;
		}

		public FileInfo GetTrack (int id)
		{
			TrackListDatabaseModel model = (TrackListDatabaseModel)library.TrackModel;
			TrackInfo track = model.GetValue (id);

			return new FileInfo (track.Uri.LocalPath);
		}
	}
}
