//
//  Util.cs
//
//  Copyright (c) Lukas Lipka, 2007.
//

using System;
using System.IO;
using System.Net;
using System.Threading;
using System.Collections;
using System.Collections.Generic;

namespace Banshee.Webcast {

	public class Util {

		private const char WILDCARD_STRING = '*';

		private Util ()
		{
		}

		public static bool GlobMatch (string pattern, string str)
		{
			if (pattern == null || str == null)
				return false;
			else if (pattern == "*")
				return true;
			else if (str == String.Empty && pattern != String.Empty)
				return false;
			else if (pattern.IndexOf ('*') == -1)
				return pattern == str;
			else
				return WildcardEquals (pattern, 0, str, 0);
		}
		
		private static bool WildcardEquals (string pattern, int patternIdx, string text, int stringIdx)
		{
			int p = patternIdx;
			
			for (int s = stringIdx; ; ++p, ++s) {
				// End of string yet?
				bool sEnd = (s >= text.Length);
				// End of pattern yet?
				bool pEnd = (p >= pattern.Length);

				// If we're looking at the end of the string...
				if (sEnd) {
					// Assume the only thing left on the pattern is/are wildcards
					bool justWildcardsLeft = true;
					
					// Current wildcard position
					int wildcardSearchPos = p;
					// While we haven't found the end of the pattern,
					// and haven't encountered any non-wildcard characters
					while (wildcardSearchPos < pattern.Length && justWildcardsLeft) {
						// Check the character at the current position
						char wildchar = pattern[wildcardSearchPos];
						
						// If it's not a wildcard character, then there is more
						// pattern information after this/these wildcards.
						if (wildchar != WILDCARD_STRING)
							justWildcardsLeft = false;
						else
							// Look at the next character
							wildcardSearchPos++;
					}
					
					// This was a prefix wildcard search, and we've matched, so
					// return true.
					if (justWildcardsLeft) {
						return true;
					}
				}
				
				// If we've gone past the end of the string, or the pattern,
				// return false.
				if (sEnd || pEnd) {
					break;
				}
				
				if (pattern[p] == WILDCARD_STRING) {
					// Look at the character beyond the '*'.
					++p;
					// Examine the string, starting at the last character.
					for (int i = text.Length; i >= s; --i) {
						if (WildcardEquals(pattern, p, text, i)) {
							return true;
						}
					}
					break;
				}
				if (pattern[p] != text[s]) {
					break;
				}
			}
			return false;
		}
	}
}
